package com.persagy.entity;

import com.alibaba.fastjson.JSONObject;
import com.persagy.entity.v2.AlarmCondition;
import lombok.Data;

/**
 * @description:
 * @author:LuoGuangyi
 * @company:PersagyTechnologyCo.,Ltd
 * @since:2020/10/18 001814:37
 * @version:V1.0 {
 * "streamId": "",
 * "opCode": "",
 * "content": [
 * {
 * "id": "000f9609-685e-47b7-bb7d-e91d31f21f8d",
 * "objId": "Eq1101050029e8fa1f6e4c81483fbfd7c554e1cdcd2e",
 * "projectId": "Pj1101050029",
 * "itemCode": "1051",
 * "category": "Eq",
 * "level": "S",
 * "remark": "更改报警等级333",
 * "condition": {
 * },
 * "open": 0,
 * "concern": 0,
 * "userDefined": 0,
 * "createUser": "System",
 * "createTime": 1603268498000,
 * "updateUser": "systemId",
 * "updateTime": 1603372305000,
 * "valid": 1
 * }
 * ]
 * }
 **/
@Data
public class AlarmDefine {
    /**
     * 报警定义ID
     */
    private String id;
    /**
     * 报警对象ID
     */
    private String objId;
    /**
     * 对象类型
     */
    private String classCode;
    /**
     * 项目ID
     */
    private String projectId;
    /**
     * 报警对象类型
     */
    private String category;
    /**
     * 严重程度
     */
    private String level;
    /**
     * 触发条件（信息点详情）
     */
    private Condition condition;

    /**
     * 报警条目编码
     */
    private String itemCode;
    /**
     * 报警名称
     */
    private String name;
    /**
     * 备注
     */
    private String remark;
    /**
     * 屏蔽状态 1-生效 0-屏蔽
     */
    private int open;
    /**
     * 是否重点关注
     */
    private int concern;

    @Override
    public String toString() {
        return JSONObject.toJSONString(this);
    }

    /**
     * 将报警条件转换为报警定义
     *
     * @param itemCode       报警类型编码
     * @param alarmCondition 报警条件
     * @param meterId        报警对象id
     * @param projectId      项目id
     * @return 报警定义
     * @author lixing
     * @version V1.0 2021/10/22 5:59 下午
     */
    public static AlarmDefine convert2AlarmDefine(
            String itemCode, AlarmCondition alarmCondition, String meterId, String projectId) {
        AlarmDefine alarmDefine = new AlarmDefine();
        alarmDefine.setId(itemCode + "-" + meterId);
        alarmDefine.setObjId(meterId);
        alarmDefine.setProjectId(projectId);
        alarmDefine.setLevel(alarmCondition.getAlarmLevel());
        alarmDefine.setItemCode(itemCode);
        // 屏蔽状态 1-生效 0-屏蔽
        alarmDefine.setOpen(1);
        // 设置报警条件
        Condition condition = new Condition();
        condition.setInfoCode(alarmCondition.getInfoCodes());
        condition.setConfigs(JSONObject.toJSONString(alarmCondition.getTriggerFrontend()));
        JSONObject effectTime = alarmCondition.getEffectTimeDetail();
        effectTime = effectTime == null ? new JSONObject() : effectTime;
        effectTime.put("type", alarmCondition.getEffectTimeType());
        condition.setEffectTime(effectTime);
        condition.setRules(alarmCondition.getTriggerRules());
        condition.setTrigger(alarmCondition.getTriggerBackend());
        condition.setEnd(alarmCondition.getEndBackend());
        // 时间转换为秒
        condition.setTriggerUphold(alarmCondition.getTriggerUphold() * 60);
        condition.setEndUphold(alarmCondition.getEndUphold() * 60);
        alarmDefine.setCondition(condition);
        return alarmDefine;
    }
}
